//
// StormPlayer by ssjx (https://ssjx.co.uk)
//
// 03/09/2023: Tidy declarations / initialisations
// 03/09/2023: Replaced DEFINE with const

#include <cybiko.h>
#include "storm.h"

const char version[]="StormPlayer 0.9.4 (2023)";
const char url[]="https://ssjx.co.uk";

struct FileFind searcher; 
struct module_t main_module;

//struct menulist listitem[30];
//struct menulist picitem[30];

struct menulist *listitem;
struct menulist *picitem;


/**************************/

void bmp(char *,struct module_t *);
void avi(char *,struct module_t *);
void pic(struct menulist *,struct module_t *,int,int);
void txt(char *,struct module_t *,long);
void html(char *,struct module_t *);
void fli(char *,struct module_t *);
void cvc(char *,struct module_t *);
void cvi(char *,struct module_t *);
void xpic(char *,struct module_t *);

void textreader(char *,struct module_t *,long);
//void bookreader(char *,struct module_t *,long);

void videonow(char *,struct module_t *);
void spectrum(char *,struct module_t *);
void gauge(struct module_t *,long,long);

int waitinput(struct module_t *);

void outro(struct module_t *);

//
void mem_dump(char *,long);
void mem_restore(char *,long);

/**************************/

long swp(long num){
	long out;
	char *ptr_in,*ptr_out;

	ptr_in=&num;
	ptr_out=&out;

	ptr_out[0]=ptr_in[3];
	ptr_out[1]=ptr_in[2];
	ptr_out[2]=ptr_in[1];
	ptr_out[3]=ptr_in[0];

	return out;
}

int bswp(int num){
	int out;
	char *ptr_in,*ptr_out;

	ptr_in=&num;
	ptr_out=&out;

	ptr_out[0]=ptr_in[1];
	ptr_out[1]=ptr_in[0];

	return out;
}

void ucase(char *in){
	int i=0;
	do{
		if (in[i]>92 && in[i]<123){in[i]-=32;}
		i++;
	}while(in[i]!=0);
}


/**************************/

// Same as strends except check upper and lower case (providing ext=lower)
// small buffer so only useful to check extensions
int ends(char *txt,char *ext){
	int out=FALSE;

	if (strends(txt,ext)){
		out=TRUE;
	}else{
		char *temp;
		temp=(char *)calloc(20,sizeof(char));
	
		if (temp!=NULL){
			strcpy(temp,ext);
			temp[19]=0;	// To prevent ucase trashing memory (though above may do..)
			ucase(temp);	
			if (strends(txt,temp)){out=TRUE;}
		}
		free(temp);
	}

	return out;
}


void findit(char *ext,struct menulist *item,int *i){
	struct File file_info;
	char *temp;
	temp=(char *)calloc(20,sizeof(char));
	
	strcpy(temp,ext);
	
	#if DEBUG==1 
		TRACE("Scanning '%s' for files.",ext);
	#endif
	
	File_ctor( &file_info );
	
	// Lower case
	
	FileFind_ctor( &searcher, &file_info, ext );
	
	do{
		if (FileFind_next_element( &searcher )){
			#if DEBUG==1
				TRACE("%d: >%s<",*i,file_info.name);
			#endif 
			strcpy(item[*i].name,file_info.name);
			item[*i].size=file_info.size;
			*i=*i+1;
		}
	    
	}while( FileFind_has_more_elements( &searcher ) );
	
	// Upper case
	
	ucase(temp);
	
	FileFind_init( &searcher, &file_info, temp );
	
	do{
		if (FileFind_next_element( &searcher )){
			#if DEBUG==1  
				TRACE("%d: >%s<",*i,file_info.name);
			#endif 
	
			strcpy(item[*i].name,file_info.name);
			item[*i].size=file_info.size;
			*i=*i+1;
		}    
	}while( FileFind_has_more_elements( &searcher ) );
	FileFind_dtor( &searcher, LEAVE_MEMORY );
	
	File_dtor( &file_info , LEAVE_MEMORY );
	free(temp);
}

//
// findprogs
//
void findprogs(char *ext,char t,struct menulist *item,int *i){
	struct File file_info;
	struct Archive resources;
	struct Input* ptr_input;
	
	char *temp;

	temp=(char *)calloc(20,sizeof(char));
	strcpy(temp,ext);

	#if DEBUG==1 
		TRACE("Scanning '%s' for apps.",ext);
	#endif

	File_ctor( &file_info );

	// Lower case
	FileFind_ctor( &searcher, &file_info, ext );

	do{
		if (FileFind_next_element( &searcher )){
			//
			Archive_ctor( &resources, file_info.name );
			if(Archive_is_good( &resources ) ){
				ptr_input = Archive_open_Ex( &resources, "root.inf" );
				if (ptr_input!=0){
					if(Input_is_good( ptr_input )){
						char inbyte=Input_read_byte( ptr_input);
						if (inbyte==t){
							#if DEBUG==1
								TRACE("%d:Prog >%s< %c",*i,file_info.name,inbyte);
							#endif 
							strcpy(item[*i].name,file_info.name);
							item[*i].size=-1;
							*i=*i+1;
						}	
					}
					Input_dtor( ptr_input, FREE_MEMORY );
				}
			}
			Archive_dtor( &resources, FREE_MEMORY );
		}
		
	}while( FileFind_has_more_elements( &searcher ) );

	FileFind_dtor( &searcher, LEAVE_MEMORY );

	File_dtor( &file_info , LEAVE_MEMORY );
	free(temp);
}


//
// Clear the display and display a message
//

void mymess(struct module_t *ptr_main_module, char *message){
	char *dis;

	dis=DisplayGraphics_get_page_ptr(ptr_main_module->m_gfx, 0); 

	// Top 10 rows black next 90 white
	memset(dis,0xff,400);
	memset(dis+400,0,3600);

	TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx, message , 1, 10 );

	TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx, version , 1, 0 );

	DisplayGraphics_show(ptr_main_module->m_gfx);
}


int lister(struct module_t *ptr_main_module,struct menulist *item,int items){
	int sel=0,i,ext=0,lstart=0;
	int mx=items-1;
	char *temp,*dis;
	temp=(char *)calloc(50,sizeof(char));

	dis=DisplayGraphics_get_page_ptr(ptr_main_module->m_gfx, 0); 

	//Make sure our font is correct!
	Graphics_set_font(ptr_main_module->m_gfx, mini_bold_font );

	// Clear top 10 rows black
	memset(dis,0xff,400);

	TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx, version , 1, 0 );
	TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );

	if (items>9){items=9;}

	do{
		int yp=0;
		// Clear bottom 90 rows white
		memset(dis+400,0,3600);
			
		for(i=0;i<items;i++){
			yp+=10;
			if (i==sel){
				// Hightlight selected (10 rows)
				memset(dis+(40*yp),0x55,400);
			}
			
			memset(temp,0,sizeof(temp));

			if (memcmp(item[lstart+i].name,"/MP3/",5)==0){
				memcpy(temp,item[lstart+i].name+5,strlen(item[lstart+i].name)-4);
			}else{
				if (memcmp(item[lstart+i].name,"/A/",3)==0){
					memcpy(temp,item[lstart+i].name+3,strlen(item[lstart+i].name)-2);
				}else{
					strcpy(temp,item[lstart+i].name);
				}
			}
			
			// Highlight folder
			if (ends(temp,".cyf")){
				memset(dis+(40*yp),0xaa,400);
			}

			DisplayGraphics_draw_text(ptr_main_module->m_gfx,temp, 0, yp );
			
			if (item[lstart+i].size>=0){
				memset(temp,0,sizeof(temp));
					
				if (item[lstart+i].size>=1024){
					sprintf(temp,"%dk",item[lstart+i].size/1024);
				}else{
					sprintf(temp,"%db",item[lstart+i].size);
				}
				
				DisplayGraphics_draw_text(ptr_main_module->m_gfx,temp, 120, yp );
			}
		}

		DisplayGraphics_show(ptr_main_module->m_gfx);

		// ------------------------ 
			 
		switch( waitinput(ptr_main_module)){
			case KEY_LEFT:
			case KEY_O:
			case KEY_ESC:
				ext=1;sel=-1;lstart=0;
			break;

			case KEY_UP:
			case KEY_Q:
				sel--;
				if(sel<0 && lstart>0){lstart--;}
				if (sel<0){sel=0;}
			break; 

			case KEY_TAB:
			case KEY_A:
			case KEY_DOWN:
				if ((lstart+sel)<mx){
					sel++;
					if(sel>8){lstart++;sel=8;}
				}
			break;

			case KEY_RIGHT:
			case KEY_ENTER:
			case KEY_P:
				ext=1;
			break;
		}
	// -------------

	}while(ext==0);

	#if DEBUG==1
		if ((sel+lstart)!=-1){
			TRACE("Selected: #%d %s",(sel+lstart),item[(sel+lstart)]);
		}else{
			TRACE("Selected: Exit/Back");
		}
	#endif

	free(temp);
	return (sel+lstart);	
}

/**************************/

long main(int argc, char *argv[],bool start){
	int menu,picstart,picend,i;

	#if DEBUG==1 
		TRACE("DEBUG VERSION");
		TRACE("short: %d int: %d long: %d",sizeof(short),sizeof(int),sizeof(long));
	#else
		TRACE("STANDARD VERSION");
	#endif

	init_module( &main_module );

	Graphics_set_font( main_module.m_gfx, mini_bold_font );

	//struct menulist listitem[30];

	listitem=(struct menulist *)calloc(MAXLIST,sizeof(struct menulist));

	do{
		int maxf=0;
		
		// DRAW SCREEN
		strcpy(listitem[0].name,"Image");
		strcpy(listitem[1].name,"Video");
		strcpy(listitem[2].name,"Text");
		strcpy(listitem[3].name,"Browser");
		strcpy(listitem[4].name,"Book Reader");
		strcpy(listitem[5].name,"Games");
		strcpy(listitem[6].name,"Applications");
		strcpy(listitem[7].name,"Exit");

		for(i=0;i<8;i++){
			listitem[i].size=-1;
		}

		menu=lister(&main_module,listitem,8);

		// Search for cybiko compressed folders
		/*
		if (menu>=0 && menu<5)
		{
			findit("/MP3/*.cyf",listitem,&maxf);
			findit("/A/*.cyf",listitem,&maxf);
			findit("*.cyf",listitem,&maxf);
		}
		*/

		switch(menu){
			case 0:
				maxf=0;
				// Image
				mymess(&main_module,"Searching for images.");
				findit("/MP3/*.bmp",listitem,&maxf);
				findit("/MP3/*.cvi",listitem,&maxf);
				findit("/MP3/*.scr",listitem,&maxf);
				findit("/MP3/*.xpic",listitem,&maxf);

				mymess(&main_module,"Searching for images..");
				findit("/A/*.bmp",listitem,&maxf);
				findit("/A/*.cvi",listitem,&maxf);
				findit("/A/*.scr",listitem,&maxf);
				findit("/A/*.xpic",listitem,&maxf);

				mymess(&main_module,"Searching for images...");
				findit("*.bmp",listitem,&maxf);
				findit("*.cvi",listitem,&maxf);
				findit("*.scr",listitem,&maxf);
				findit("*.xpic",listitem,&maxf);

				picstart=maxf;
					mymess(&main_module,"Searching for pics.");
					findit("/MP3/*.pic",listitem,&maxf);

					mymess(&main_module,"Searching for pics..");
					findit("/A/*.pic",listitem,&maxf);

					mymess(&main_module,"Searching for pics...");
					findit("*.pic",listitem,&maxf);
				picend=maxf;

			break;

			case 1:
				// Video
				mymess(&main_module,"Searching for videos.");
				findit("/MP3/*.avi",listitem,&maxf);
				findit("/MP3/*.cvc",listitem,&maxf);
				findit("/MP3/*.flc",listitem,&maxf);
				findit("/MP3/*.fli",listitem,&maxf);
				findit("/MP3/*.wav",listitem,&maxf);

				mymess(&main_module,"Searching for videos..");

				findit("/A/*.avi",listitem,&maxf);
				findit("/A/*.cvc",listitem,&maxf);
				findit("/A/*.flc",listitem,&maxf);
				findit("/A/*.fli",listitem,&maxf);
				findit("/A/*.wav",listitem,&maxf);

				mymess(&main_module,"Searching for videos...");

				findit("*.avi",listitem,&maxf);
				findit("*.cvc",listitem,&maxf);
				findit("*.flc",listitem,&maxf);
				findit("*.fli",listitem,&maxf);
				findit("*.wav",listitem,&maxf);
			break;

			case 2:
				// Text
				mymess(&main_module,"Searching for texts.");
				findit("/MP3/*.txt",listitem,&maxf);
				findit("/MP3/*.asc",listitem,&maxf);

				mymess(&main_module,"Searching for texts..");
				findit("/A/*.txt",listitem,&maxf);
				findit("/A/*.asc",listitem,&maxf);

				mymess(&main_module,"Searching for texts...");
				findit("*.txt",listitem,&maxf);
				findit("*.asc",listitem,&maxf);
			break;


			case 3:
				// Browser
				mymess(&main_module,"Searching for files.");

				findit("/MP3/*.htm",listitem,&maxf);
				findit("/MP3/*.html",listitem,&maxf);

				mymess(&main_module,"Searching for files..");
				findit("/A/*.htm",listitem,&maxf);
				findit("/A/*.html",listitem,&maxf);

				mymess(&main_module,"Searching for files...");
				findit("*.htm",listitem,&maxf);
				findit("*.html",listitem,&maxf);
			break;


			case 4:
				// Book
				mymess(&main_module,"Searching for files.");

				findit("/MP3/*.txt",listitem,&maxf);
				findit("/MP3/*.asc",listitem,&maxf);
				findit("/MP3/*.book",listitem,&maxf);

				mymess(&main_module,"Searching for files..");
				findit("/A/*.txt",listitem,&maxf);
				findit("/A/*.asc",listitem,&maxf);
				findit("/A/*.book",listitem,&maxf);

				mymess(&main_module,"Searching for files...");
				findit("*.txt",listitem,&maxf);
				findit("*.asc",listitem,&maxf);
				findit("*.book",listitem,&maxf);
			break;

			case 5:
				// Games
				mymess(&main_module,"Searching for games.");
				findprogs("/MP3/*.app",'g',listitem,&maxf);

				mymess(&main_module,"Searching for games..");
				findprogs("/A/*.app",'g',listitem,&maxf);

				mymess(&main_module,"Searching for games...");
				findprogs("*.app",'g',listitem,&maxf);
			break;

			case 6:
				// Apps
				mymess(&main_module,"Searching for apps.");
				findprogs("/MP3/*.app",'a',listitem,&maxf);

				mymess(&main_module,"Searching for apps..");
				findprogs("/A/*.app",'a',listitem,&maxf);

				mymess(&main_module,"Searching for apps...");
				findprogs("*.app",'a',listitem,&maxf);
			break;

			case 7:
			case -1:
				//mymess(&main_module,"Thanks for using StormPlayer!");

				#if DEBUG==1
					TRACE("Exiting...");
				#endif

				menu=-1;
			break;
		}

		#if DEBUG==1
			TRACE("%d files found",maxf);
		#endif

		if (menu>=0){
			long myfilesize=0;
			char myfile[50];
			int choice;
			
			while((choice=lister(&main_module,listitem,maxf))!=-1){
				strcpy(myfile,listitem[choice].name);
				myfilesize=listitem[choice].size;

				//
				// Copy Menulist to a file and free() listitem.
				//
				mem_dump(listitem,sizeof(struct menulist)*MAXLIST);
				
				// the pic viewer makes a copy of this so we need to keep it!
				if( ends(myfile, ".pic" )==FALSE ){
					free(listitem);
				}
				
				// Compressed folder
				//if( ends(myfile , ".cyf" ) )
				//{
				// Get folder contents
				// Display contents if more than one file
				// Get file name
				//}

				// Image
				if( ends(myfile , ".bmp" ) ){bmp(myfile,&main_module);}
				if( ends(myfile , ".cvi" ) ){cvi(myfile,&main_module);}
				if( ends(myfile , ".scr" ) ){spectrum(myfile,&main_module);}
				if( ends(myfile , ".xpic" ) ){xpic(myfile,&main_module);}
				
				if( ends( myfile, ".pic" ) ){
					int picmax,mypic;
					picitem=(struct menulist *)calloc((picend-picstart+1),sizeof(struct menulist));
							
					if (picitem!=0){
						mypic=-1;
						picmax=0;
						for(i=picstart;i<picend;i++){
							strcpy(picitem[picmax].name,listitem[i].name);

							// Get the position of the first pic to show
							if (mypic==-1){
								if (strcmp(myfile,picitem[picmax].name)==0){
									mypic=picmax;				
								}	
							}
							
							picmax++;
						}
						picmax-=1;
					}else{
						#if DEBUG==1
							TRACE("Picitem memory error. Tried to alloc %ld blocks",(picend-picstart+1));
						#endif
					}

					#if DEBUG==1
						TRACE("S: %ld E: %ld File : %s Mypic : %ld  Picmax: %ld:",picstart,picend,myfile,mypic,picmax);
					#endif

					pic(picitem,&main_module,mypic,picmax);
					free(picitem);
				}

				// Video
				if( ends(myfile, ".avi" ) ){avi(myfile,&main_module);}
				if( ends(myfile, ".fli" ) ){fli(myfile,&main_module);}
				if( ends(myfile, ".flc" ) ){fli(myfile,&main_module);}
				if( ends(myfile, ".cvc" ) ){cvc(myfile,&main_module);}
				if( ends(myfile, ".wav" ) ){videonow(myfile,&main_module);}

				// Text
				if (menu==2){
					if(ends( myfile, ".txt" ) ){txt(myfile,&main_module,myfilesize);}
					if(ends( myfile, ".asc" ) ){txt(myfile,&main_module,myfilesize);}
				}

				// Browser
				if( ends( myfile, ".htm" )||ends( myfile, ".html" ) ){html(myfile,&main_module);}

				// Book
				if (menu==4){
					if( ends(myfile, ".txt" ) ){textreader(myfile,&main_module,myfilesize);}
					if( ends(myfile, ".asc" ) ){textreader(myfile,&main_module,myfilesize);}
					if( ends(myfile, ".book" ) ){textreader(myfile,&main_module,myfilesize);}
				}

				// Games
				if (menu==5 || menu==6){
					if (!start_module( myfile ) ){
						TRACE("Could not run %s",myfile);
					}
				}

				// Load and restore the menulist
				listitem=(struct menulist *)calloc(MAXLIST,sizeof(struct menulist));
				mem_restore(listitem,sizeof(struct menulist)*MAXLIST);
			}
		}
		
	}while(menu>=0);

	free(listitem);


	//
	// Outro
	//

	
	Graphics_set_font(main_module.m_gfx, mini_bold_font );
	outro(&main_module);


	TRACE("Thanks for using StormPlayer!");
	return 0L;
}
