//
// General Graphics Functions
//
// 02/09/2023: Tidy declarations / initialisations
// 02/09/2023: Ordered dither should be quicker
// 02/09/2023: Removed variable from Snapshot
//

#include <cybiko.h>

void snapshot(char *);

int getinput(struct module_t *ptr_main_module){
	struct Message* ptr_message;
	struct KeyParam* ptr_key_param;
	int num=0;

	ptr_message = cWinApp_get_message( ptr_main_module->m_process, 1, 1,MSG_USER );
		
	if(ptr_message->msgid==MSG_KEYDOWN)	{
		ptr_key_param = Message_get_key_param( ptr_message );

		num=ptr_key_param->scancode;
	}    

	Message_delete(ptr_message);

	return num;
}

int waitinput(struct module_t *ptr_main_module){
	struct Message* ptr_message;
	struct KeyParam* ptr_key_param;
	int num=0,exit=0;

	// Clear the message buffer of key downs presses
	while(ptr_message = cWinApp_peek_message( ptr_main_module->m_process, TRUE, MSG_KEYDOWN, MSG_KEYDOWN )){
		Message_delete( ptr_message );
		// TRACE("Deleting message");
	}

	do{
		ptr_message = cWinApp_get_message( ptr_main_module->m_process, 0, 1,MSG_USER );
			
		if(ptr_message->msgid==MSG_KEYDOWN){
			ptr_key_param = Message_get_key_param( ptr_message );  
			num=ptr_key_param->scancode;
			exit=1;
		}    
		
		Message_delete(ptr_message);
	}while(exit==0);

	if (num==KEY_S){
		snapshot(DisplayGraphics_get_page_ptr(ptr_main_module->m_gfx, 0));
	}

	return num;
}



//Converts an 8bit row to a 2bit (4ppb) row
void conv_row(char *out,char *in,long width,long j){
	long px=0;
	int i;
	int aw=(width & 0xfffc)>>2;
	long apos=(aw*(j-1));

	for(i=0;i<aw;i++){
		char p1=3-((in[px++] & 0xff)>>6);
		char p2=3-((in[px++] & 0xff)>>6);
		char p3=3-((in[px++] & 0xff)>>6);
		char p4=3-((in[px++] & 0xff)>>6);

		out[apos++]=(p1<<6)+(p2<<4)+(p3<<2)+p4;
	}
}

/*
Converts 24bit to 2bit
*/

char gbyte24(char r,char g,char b){
	long av=(r & 0xff)+(g & 0xff)+(b & 0xff);
	av=(av/3)>>6;
	return (3-av);
}

/*
Converts 24bit to 8bit
*/

char grey24(char r,char g,char b){
	long av=(r & 0xff)+(g & 0xff)+(b & 0xff);
	return (av/3);
}


/*
Converts 16bit to 2bit
*/

char gbyte16(short col){
	char r=((col & 0x7c00)>>10);
	char g=((col & 992)>>5);
	char b=(col & 31);
	char av=((r+g+b)/3)>>3;
	return (3-av);
}


/*
Plots a pixel directly to the screen
*/


void fplot(char *scr,long x,long y,long w,char c){
	long ax=(x & 0xfffc)>>2;
	long aw=(w & 0xfffc)>>2;

	long ay=y*aw;

	//3- Flip order (<<2 = *4)
	long bx=3-(x-(ax<<2));

	char byt=c<<(bx<<1);
	char clr=0x03<<(bx<<1);

	scr[ax+ay]=(scr[ax+ay]&~clr)+byt;
}

/*
Gauge
Shown at the bottom so we can safely use the top of the screen as a buffer!
*/


void gauge(struct module_t *ptr_main_module,long cur,long mx){
	long stp=(cur*100)/mx;
	char *temp;
	temp=(char *)calloc(20,sizeof(char));

	sprintf(temp,"Loading... %ld%%",stp);

	TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );
	TGraph_fill_rect(ptr_main_module->m_gfx,0,90,160,10);

	TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE);
	DisplayGraphics_draw_text(ptr_main_module->m_gfx,temp, 1, 90 );

	DisplayGraphics_show(ptr_main_module->m_gfx);
	free(temp);
}

/*
Screen Zoom 80x50 -> 160x100 
*/


void zoom(char *in,char *out,int width,int height,int ox,int oy){
	int i,j,row=0,cnt=0;
	int pos=((width>>2)*oy)+ox;

	for(j=0;j<50;j++){
		for(i=0;i<20;i++){
			// one byte in...
			char a=(in[pos+i] & 0xc0)>>6;
			char b=(in[pos+i] & 0x30)>>4;
			char c=(in[pos+i] & 0x0c)>>2;
			char d=(in[pos+i] & 0x03);

			// ...two bytes out
			out[cnt++]=(a<<6)|(a<<4)|(b<<2)|b;
			out[cnt++]=(c<<6)|(c<<4)|(d<<2)|d;
		}

		//copy line down one
		memcpy(out+row+40,out+row,40);
		row+=80;
		
		// move down one line (display)
		cnt+=40;
		
		// move down one line
		pos+=(width>>2);
	}
}


void center(struct module_t *ptr_main_module,char *text,int y){
	int width=Font_string_width( DisplayGraphics_get_font( ptr_main_module->m_gfx),text );
	int pos=(160-width)>>1;	    
	if (pos<0){pos=0;}

	TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos+1, y+1 );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos-1, y+1 );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos+1, y-1 );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos-1, y-1 );

	TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );
	DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos, y );
}

//
// Ordered dithering
//


	char pattern[8][8] = {
    { 0, 32,  8, 40,  2, 34, 10, 42},   // 8x8 Bayer ordered dithering
    {48, 16, 56, 24, 50, 18, 58, 26},   // pattern.  Each input pixel
    {12, 44,  4, 36, 14, 46,  6, 38},   // is scaled to the 0..63 range
    {60, 28, 52, 20, 62, 30, 54, 22},   // before looking in this table
    { 3, 35, 11, 43,  1, 33,  9, 41},   // to determine the action.
    {51, 19, 59, 27, 49, 17, 57, 25},
    {15, 47,  7, 39, 13, 45,  5, 37},
    {63, 31, 55, 23, 61, 29, 53, 21} };

void ordered(char *out,char *in,int width,int yp){
	long px=0;
	int x,j,i=0;
	int	aw=(width>>2);
	long apos=aw*(yp-1);
	
	// Get the correct pattern row
	if (yp>7){
		j=yp % 8; 
	}else{
		j=yp;
	}

	// dither row
	for(x=0;x<width;x++){
		char c=(in[x] & 0xff)>>2;
		if (c<pattern[i][j]){in[x]=3-(c>>4);}else{in[x]=0;}

		i++;
		if (i>7){i=0;}
	}
	//

	for(x=0;x<aw;x++){
		char p1=in[px++];
		char p2=in[px++];
		char p3=in[px++];
		char p4=in[px++];
		out[apos++]=(p1<<6)+(p2<<4)+(p3<<2)+p4;
	}
}


void snapshot(char *in){
	struct FileOutput file_output;
	const long hd1=0x0201a064;
	const long hd2=0x0000a064;
	
	char *fn,i=0;
	fn=(char *)calloc(30,sizeof(char));

	FileOutput_ctor( &file_output );

	// Loop until we find an unused snapshot name!
	for(;;){
		memset(fn,0,30);
		sprintf(fn,"stormshot%d.pic",i);
		if(File_exists(fn)) {i++;}else{break;}
	}

	if( FileOutput_open( &file_output, fn, TRUE ) ){
		FileOutput_write( &file_output,&hd1,4);
		FileOutput_write( &file_output,&hd2,4);
		FileOutput_write( &file_output,in,4000);
	}

	FileOutput_dtor( &file_output, LEAVE_MEMORY );
	TRACE("Snapshot Taken, saved as %s",fn);
	free(fn);
}


void outro(struct module_t *ptr_main_module){
	char i;
	char *txt[7];
	txt[0]="Thanks for playing!";
	txt[1]="";
	txt[2]="Please visit:";
	txt[3]="";
	txt[4]="https://ssjx.co.uk";
	txt[5]="";
	txt[6]="For more Cybiko stuff!";
	
	// Cool shadow
	TGraph_set_color(ptr_main_module->m_gfx, CLR_LTGRAY );
	TGraph_fill_rect(ptr_main_module->m_gfx,14,14,140,80);

	// Draw on top
	TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );
	TGraph_fill_rect(ptr_main_module->m_gfx,10,10,140,80);

	TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE );
	TGraph_fill_rect(ptr_main_module->m_gfx,10+1,10+1,140-2,80-2);

	// Draw Title
	TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );

	// Draw text
	for(i=0;i<7;i++){
		center(ptr_main_module, txt[i],12+(i*10) );
	}

	DisplayGraphics_show(ptr_main_module->m_gfx);

	// Wait to exit
	waitinput(ptr_main_module);
}
